% This script demonstrates the use of the "sampleGMM" function. It
% generates a number of samples from a Gaussian mixture model with
% specified parameters using a single function call. For a number of
% dimensions smaller or equal 3, the samples are subsequently visualized.

%%% clean up %%%
clear; clc; close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                             % Settings %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nSamples    = 1000;     % number of samples to be generated
nComponents = 5;        % number of mixture components
nDims       = 2;        % dimensionality of the samples
variance    = 0.1;      % variance of the (isotropic) components


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                             % Sampling %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% generate random mixture components %%% 
mus     = randn(nComponents, nDims);
sigmas  = variance * repmat(eye(nDims), 1, 1, nComponents);

%%% generate random mixture weights %%%
weights = rand(nComponents, 1);
weights = weights / sum(weights);

%%% generate the samples %%%
[samples, indicators] = sampleGMM(mus, sigmas, weights, nSamples);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                             % Visualization %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% visualization only possible for up to three dimensions %%%
if nDims > 3
    disp('Visualization only possible for up to three dimensions.')
    return
end

%%% figure environment %%%
figure(), hold on, grid on

%%% plot the samples of each mixture component %%%
for c = 1:nComponents
    
    %%% corresponding samples %%%
    inds = indicators == c;
    
    %%% call corresponding plotting routine %%%
    switch nDims
        case 1
            stem(samples(inds), c * ones(sum(inds), 1));
            set(gca, 'ytick', [])
        case 2
            scatter(samples(inds,1), samples(inds,2));
            view(2)
        case 3
            scatter3(samples(inds,1), samples(inds,2), samples(inds,3));
            view(3)
    end
end

%%% annotate plot %%%
xlabel('first dimension')
if nDims >= 2
    ylabel('second dimension')
    if nDims == 3
        zlabel('third dimension')
    end
end
title(sprintf('Samples from a Gaussian mixture model with %d components', nComponents))